Imports System.Web.UI
Imports System.Web.UI.WebControls
Imports System.Net
Imports System.IO
Imports System.Configuration
Imports System.Text

Namespace Derrick.Web
	Partial Public Class CometProxy
		Inherits System.Web.UI.Page

		Private _proxySwitch As New TraceSwitch("CometProxy", "CometProxy")

		Protected Sub Page_Load(ByVal sender As Object, ByVal e As EventArgs)
            System.Diagnostics.Trace.WriteLineIf(_proxySwitch.TraceVerbose, "Entering CometProxy.Page_Load.")
			Dim bytes() As Byte
			Using reader As Stream = Request.GetBufferlessInputStream
                bytes = New Byte(CInt(reader.Length - 1)) {}
				reader.Read(bytes, 0, CInt(Fix(reader.Length)))
			End Using

            System.Diagnostics.Trace.WriteLineIf(_proxySwitch.TraceVerbose, "Building HTTP request to Comet.")
			Dim newUrl As String = ConfigurationManager.AppSettings("CometServer")
			Dim cometRequest As HttpWebRequest = CType(HttpWebRequest.Create(newUrl), HttpWebRequest)

			cometRequest.SendChunked = False
			cometRequest.KeepAlive = True
            'cometRequest.Credentials = New NetworkCredential("server", "server123")
			cometRequest.Method = Request.HttpMethod
			cometRequest.ContentType = Request.ContentType
			cometRequest.UserAgent = Request.UserAgent
			If bytes.Length > 0 Then
				cometRequest.ContentLength = bytes.Length
			Else
				cometRequest.ContentLength = 0
			End If

			Dim containsOriginHeader As Boolean = False
            System.Diagnostics.Trace.WriteLineIf(_proxySwitch.TraceVerbose, "Checking for Origin header.")

			For i  = 0 To Request.Headers.Count - 1
				If Request.Headers.GetKey(i).Equals("Origin") Then
					containsOriginHeader = True
					Exit For
				End If
			Next i

			If containsOriginHeader Then
                'cometRequest.Headers"Origin" = Request.Headers"Origin"
				cometRequest.Headers.Set("Origin", Request.Headers("Origin"))
			Else
				cometRequest.Headers.Add("Origin", Request.Url.AbsoluteUri)
			End If

            System.Diagnostics.Trace.WriteLineIf(_proxySwitch.TraceVerbose, "Adding Origin header.")

			Dim stream As Stream = Nothing

			If cometRequest.ContentLength > 0 AndAlso (Not cometRequest.Method.Equals("OPTIONS")) Then
				stream = cometRequest.GetRequestStream
				stream.Write(bytes, 0, bytes.Length)
			End If

			If stream IsNot Nothing Then
				stream.Close()
			End If

            'Console.WriteLine(System.Text.Encoding.ASCII.GetString(bytes))
            System.Diagnostics.Trace.WriteLineIf(_proxySwitch.TraceVerbose, "Forwarding message: " & System.Text.Encoding.ASCII.GetString(bytes))

			Try
				Response.ClearHeaders()
				Dim res As HttpWebResponse = CType(cometRequest.GetResponse, HttpWebResponse)
				For i  = 0 To res.Headers.Count - 1
					Dim headerName As String = res.Headers.GetKey(i)
                    'Response.Headers.Set(headerName, res.HeadersheaderName)
					Response.AddHeader(headerName, res.Headers(headerName))
				Next i

                System.Diagnostics.Trace.WriteLineIf(_proxySwitch.TraceVerbose, "Added headers.")

				Dim s As Stream = res.GetResponseStream

				Dim msgSizeStr As String = ConfigurationManager.AppSettings("MessageSize")
				Dim messageSize As Integer = Convert.ToInt32(msgSizeStr)

				Dim read(messageSize - 1) As Byte
				' Reads 256 characters at a time.    
				Dim count As Integer = s.Read(read, 0, messageSize)
				Do While count > 0
					' Dumps the 256 characters on a string and displays the string to the console.
					Dim actualBytes(count - 1) As Byte
					Array.Copy(read, actualBytes, count)
                    'Response.BinaryWrite(actualBytes)
                    'Dim cometResponseStream As String = Encoding.UTF8.GetString(actualBytes)
					Dim cometResponseStream As String = Encoding.ASCII.GetString(actualBytes)
                    'Response.Write(actualBytes)
					Response.Write(cometResponseStream)
					count = s.Read(read, 0, messageSize)
				Loop

				Response.End()
                System.Diagnostics.Trace.WriteLineIf(_proxySwitch.TraceVerbose, "Sent Message.")

				s.Close()
			Catch ex As Exception
                'Console.WriteLine(e.Message)
                System.Diagnostics.Trace.WriteLine("CometProxy Exception: " & ex.ToString)
			End Try

            System.Diagnostics.Trace.WriteLineIf(_proxySwitch.TraceVerbose, "Leaving CometProxy.Page_Load.")
            System.Diagnostics.Trace.Flush()
            System.Diagnostics.Trace.Close()
		End Sub
	End Class
End Namespace
