Imports System.Threading
Imports System.Text
Imports System.Net
Imports System.Net.Sockets
Imports System.Xml.Serialization
Imports System.IO
Imports System.Web.Script.Serialization
Imports System.Collections.Specialized

Namespace Derrick.Web
	Public Enum MessageFormat
		text = 0
		xml = 1
		json = 2
		querystring = 3
	End Enum

	Public Class Client
        Private client_two As TcpClient = Nothing
		Private stream As NetworkStream = Nothing
		Private authconfig As AuthConfig

		Private server As Server

		Private read_buffer(1023) As Byte
		Private buffer As String = ""
		Private bufferpos As Integer = 0

        Private request_two As Request
		Private state As ParseState = ParseState.Start

		Public IsLive As Boolean = True

		Private _traceSwitch As TraceSwitch

		Private _lastUpdate As Date

		Public ReadOnly Property TCPClient As TcpClient
			Get
                Return client_two
			End Get
		End Property

		Protected _isTcpClientClosed As Boolean
		Private Shared _tcpClientMutex As New Mutex

		Public Property ID As Integer

		Public Property SendMessage As Boolean

		Private _authenticated As Boolean
		Private _messageFormat As MessageFormat
		Private _callbackValue As String

		Private _nameValuePairs As New NameValueCollection

		Public ReadOnly Property Request As Request
			Get
                Return request_two
			End Get
		End Property

		Public Sub New(ByVal c As TcpClient, ByVal s As Server, ByVal a As AuthConfig, ByVal id As Integer)
			_traceSwitch = New TraceSwitch("TraceLevelSwitch", "COMET Client.")
			Me.ID = id

			Try
                client_two = c
				server = s
				authconfig = a

				_lastUpdate = Date.Now
				_messageFormat = MessageFormat.json

				Dim tcpClientClosed As Boolean = False
				_tcpClientMutex.WaitOne()
				tcpClientClosed = _isTcpClientClosed
				_tcpClientMutex.ReleaseMutex()
				If tcpClientClosed Then
					Exit Sub
				End If
                stream = client_two.GetStream

				stream.BeginRead(read_buffer, 0, 1024, New AsyncCallback(AddressOf callbackRead), Me)
			Catch ex As Exception
				Trace.WriteLineIf(_traceSwitch.TraceError, ex.ToString, "COMET Client")
			End Try
		End Sub

		Public Shared Sub callbackRead(ByVal ar As IAsyncResult)
			Dim localTraceSwitch As New TraceSwitch("TraceLevelSwitch", "COMET client")
			Trace.WriteLineIf(localTraceSwitch.TraceVerbose, "Inside Client.callbackRead...")

			Dim dc As Client = CType(ar.AsyncState, Client)
			Dim bytes As Integer = 0

			Try
				bytes = dc.stream.EndRead(ar) 'this was called when connection was closed!

                ' If bytes <= 0 Then

                'dc.CleanUp
                ' Return

                'End If

                dc.buffer &= Encoding.ASCII.GetString(dc.read_buffer, 0, bytes)

                'Console.WriteLine(dc.buffer)

                dc.ParseInput()

                If dc.stream IsNot Nothing Then
                    dc.stream.BeginRead(dc.read_buffer, 0, 1024, New AsyncCallback(AddressOf Client.callbackRead), dc)
                End If
            Catch e As Exception
                'Console.WriteLine("Client: Error in callbackRead() - {0}", e.Message)
                Trace.WriteLineIf(localTraceSwitch.TraceError, e.ToString, "Derrick COMET Client")
            End Try

			Trace.WriteLineIf(localTraceSwitch.TraceVerbose, "Exiting Client.callbackRead.")
		End Sub

		Private Enum ParseState
			Start
			Headers
			Content
			Done
		End Enum

		Private Sub ParseInputOriginal()
			Dim pos As Integer
			Dim temp As String
			Dim skip As Boolean = False

			Trace.WriteLineIf(_traceSwitch.TraceVerbose, "Inside ParseInput...")

			Do While bufferpos < buffer.Length - 1 AndAlso state <> ParseState.Done AndAlso Not skip
				Select Case state
					Case ParseState.Start
						Trace.WriteLineIf(_traceSwitch.TraceVerbose, "Inside ParseInput(): ParseState.Start")
						pos = buffer.IndexOf(vbCrLf, bufferpos)
						If pos >= 0 Then
							temp = buffer.Substring(bufferpos, pos - bufferpos)
							bufferpos = pos + 2

							Dim parts() As String = temp.Split(" "c)
							If parts.Length = 3 Then
                                request_two = New Request

                                request_two.Method = parts(0)
                                request_two.Url = parts(1)
                                request_two.Version = parts(2)

								state = ParseState.Headers
							Else
								CleanUp()
							End If
						End If
					Case ParseState.Headers
						Trace.WriteLineIf(_traceSwitch.TraceVerbose, "Inside ParseInput(): ParseState.Headers")
						pos = buffer.IndexOf(vbCrLf, bufferpos)
						If pos >= 0 Then
							temp = buffer.Substring(bufferpos, pos - bufferpos)
							bufferpos = pos + 2

							If temp.Length > 0 Then
								Dim parts() As String = temp.Split(New String(0) { ": " }, StringSplitOptions.RemoveEmptyEntries)
								If parts.Length = 2 Then
                                    request_two.Headers.Add(parts(0), parts(1))
								End If
							Else
                                If request_two.HasContent Then
                                    state = ParseState.Content
                                Else
                                    state = ParseState.Done
                                End If
							End If
						End If
					Case ParseState.Content
						Trace.WriteLineIf(_traceSwitch.TraceVerbose, "Inside ParseInput(): ParseState.Content")
                        If bufferpos + request_two.ContentLength <= buffer.Length Then
                            Dim t As Integer = bufferpos + request_two.ContentLength - 1
                            request_two.Body = buffer.Substring(bufferpos, request_two.ContentLength)
                            bufferpos += request_two.ContentLength

                            state = ParseState.Done
                        Else
                            skip = True
                        End If
				End Select
			Loop

			If state = ParseState.Done Then
				Trace.WriteLineIf(_traceSwitch.TraceVerbose, "Inside ParseInput(): ParseState.Done")
				state = ParseState.Start

                'Response response

				'if (request.VerifyAuth(authconfig))
                '
                '    Trace.WriteLineIf(_traceSwitch.TraceVerbose, "Inside ParseInput(): calling server.HandleRequest")
                '    response = server.HandleRequest(request, _lastUpdate, this)
                '    Trace.WriteLineIf(_traceSwitch.TraceVerbose, "Inside ParseInput(): Done server.HandleRequest")
                'End if
				'else
                '
                '    Trace.WriteLineIf(_traceSwitch.TraceVerbose, "Client.ParseInput...ParseState.Done but unauthorized.")
                '    response = Response.Get401Response(authconfig)
                '    SendMessage = true
                'end else

				'if (SendMessage)
                '
                '    Trace.WriteLineIf(_traceSwitch.TraceVerbose, "Client.ParseInput...Sending message to client now.")
                '    response.SendResponse(stream, this)
                '    _lastUpdate = response.LastUpdate
                'end if

                'request = null

                If request_two.VerifyAuth(authconfig) Then
                    Trace.WriteLineIf(_traceSwitch.TraceVerbose, "This client has been authenticated.")
                    _authenticated = True
                Else
                    Trace.WriteLineIf(_traceSwitch.TraceVerbose, "This client failed authentication.")
                    Dim response As Response = response.Get401Response(authconfig)
                    response.SendResponse(stream, Me)
                End If
			End If
		End Sub

		Public Sub Send(ByVal data As String)
			Dim bytes() As Byte = Encoding.ASCII.GetBytes(data)
			Dim offset As Integer = 0
			Dim len As Integer = 0
            'Console.WriteLine("Sending: " & bytes.Length)
			Do While offset < bytes.Length
				offset = Math.Min(offset, bytes.Length - 1)
				len = Math.Min(1024, bytes.Length - offset)

				stream.Write(bytes, offset, len)

				offset += 1024
			Loop
            'Console.WriteLine("Done.")
		End Sub

		Public Sub CleanUp()
			Try
				stream.Close()
			Catch ex As Exception
				Trace.WriteLineIf(_traceSwitch.TraceVerbose, "Error in Client.CleanUp on stream.Close: " & ex.ToString)
			End Try
			Try
				stream.Dispose()
			Catch ex As Exception
				Trace.WriteLineIf(_traceSwitch.TraceVerbose, "Error in Client.CleanUp on stream.Dispose: " & ex.ToString)
			End Try
			Try
				_tcpClientMutex.WaitOne()
				_isTcpClientClosed = True
                client_two.Close()
				_tcpClientMutex.ReleaseMutex()
			Catch ex As Exception
				Trace.WriteLineIf(_traceSwitch.TraceVerbose, "Error in Client.CleanUp on client.CLose: " & ex.ToString)
			End Try

			stream = Nothing
            client_two = Nothing

			IsLive = False
		End Sub

		Public Sub SendResponse(ByVal args As SendMessageEventArgs)
			Trace.WriteLineIf(_traceSwitch.TraceVerbose, "Client.SendResponse(args) called...")

            If args Is Nothing OrElse args.Timestamp.ToString Is Nothing Then
                Exit Sub
            End If
			If _lastUpdate > args.Timestamp Then
				Exit Sub
			End If

			Dim errorInSendResponse As Boolean = False

			Dim jsonSerializer As JavaScriptSerializer = Nothing

			Try
				jsonSerializer = New JavaScriptSerializer
			Catch ex As Exception
				errorInSendResponse = True
				Trace.WriteLine("Cannot instantiate JSON serializer: " & ex.ToString)
			End Try

			If errorInSendResponse Then
				Trace.WriteLineIf(_traceSwitch.TraceVerbose, "Unable to instantiate JavaScriptSerializer object.  Exiting Client.SendResponse")
				Exit Sub
			End If

			Dim jsonMessageToSend As String
			Dim response As Response

			If Nothing Is request Then
				Trace.WriteLineIf(_traceSwitch.TraceVerbose, "Request object is null.  Exiting Client.SendResponse")
				Exit Sub
			End If

            If request_two.HasContent Then
                If _messageFormat = MessageFormat.json Then
                    Dim jsonObjectToSend As New ClientMessage3
                    jsonObjectToSend.SendTimestamp = args.Timestamp
                    jsonObjectToSend.Message = args.Message
                    jsonMessageToSend = jsonSerializer.Serialize(jsonObjectToSend)
                    response = response.GetHtmlResponse(jsonMessageToSend, args.Timestamp, _messageFormat)
                    response.SendResponse(stream, Me)
                ElseIf _messageFormat = MessageFormat.xml Then
                ElseIf _messageFormat = MessageFormat.querystring Then
                    Dim callbackMessage As String = _callbackValue & "( {'Message':'" & args.Message & "' });"
                    response = response.GetHtmlResponse(callbackMessage, args.Timestamp, _messageFormat)
                    response.SendResponse(stream, Me)
                End If

                _lastUpdate = args.Timestamp
            Else
                Trace.WriteLineIf(_traceSwitch.TraceVerbose, "The client request is missing required info.")
                Dim jsonObjectToSend As New ClientMessage3
                jsonObjectToSend.SendTimestamp = args.Timestamp
                jsonObjectToSend.Message = args.Message
                jsonMessageToSend = jsonSerializer.Serialize(jsonObjectToSend)
                response = response.GetHtmlResponse(jsonMessageToSend, args.Timestamp, _messageFormat)
                response.SendResponse(stream, Me)
            End If

			Trace.WriteLineIf(_traceSwitch.TraceVerbose, "Exiting Client.SendResponse successfully.")
		End Sub

		Public Sub ParseRequestContent()
			If _traceSwitch Is Nothing Then
				_traceSwitch = New TraceSwitch("TraceLevelSwitch", "Comet Tracing")
			End If

			Trace.WriteLineIf(_traceSwitch.TraceInfo, "Inside Client.ParseRequestContent...")

            If String.IsNullOrEmpty(request_two.Body) Then
                Trace.WriteLineIf(_traceSwitch.TraceVerbose, "No content in the body of the request!")
                Exit Sub
            End If

			Try
				Dim mem As New MemoryStream
                mem.Write(System.Text.Encoding.ASCII.GetBytes(request_two.Body), 0, request_two.Body.Length)
				mem.Seek(0, 0)

                If Not request_two.Headers.ContainsKey("Content-Type") Then
                    _lastUpdate = Date.Now
                    _messageFormat = MessageFormat.json
                    Trace.WriteLineIf(_traceSwitch.TraceVerbose, "Client message doesn't contain Content-Type header.")
                    'return
                Else
                    If request_two.Headers("Content-Type").Contains("xml") Then
                        Trace.WriteLineIf(_traceSwitch.TraceVerbose, "Received XML content from client.")
                        _messageFormat = MessageFormat.xml

                        '#Region "Process HTTP message as XML"

                        Try
                            'Picks up message from HTTP:
                            Dim s As New XmlSerializer(GetType(Derrick.Web.SIServer.SIRequest))

                            'Loads message into object for processing.
                            Dim data As Derrick.Web.SIServer.SIRequest = CType(s.Deserialize(mem), Derrick.Web.SIServer.SIRequest)
                        Catch ex As Exception
                            Trace.WriteLineIf(_traceSwitch.TraceVerbose, "During parse of client XML request got this exception: " & ex.ToString)
                        End Try

                        '#End Region ' Process HTTP message as XML
                    ElseIf request_two.Headers("Content-Type").Contains("json") Then
                        Trace.WriteLineIf(_traceSwitch.TraceVerbose, "Received json content from client.")
                        _messageFormat = MessageFormat.json

                        '#Region "Process HTTP message as JSON"

                        Try
                            Dim jsonSerializer As New JavaScriptSerializer
                            Dim clientMessage As ClientMessage3 = jsonSerializer.Deserialize(Of ClientMessage3)(request_two.Body)
                            _lastUpdate = clientMessage.SendTimestamp
                            Trace.WriteLineIf(_traceSwitch.TraceVerbose, "Received the following message: ")
                            Trace.WriteLineIf(_traceSwitch.TraceVerbose, "SendTimestamp: " & clientMessage.SendTimestamp.ToString)
                            Trace.WriteLineIf(_traceSwitch.TraceVerbose, "Browser: " & clientMessage.Browser)
                            Trace.WriteLineIf(_traceSwitch.TraceVerbose, "Message: " & clientMessage.Message)
                        Catch ex As Exception
                            Trace.WriteLineIf(_traceSwitch.TraceVerbose, "Error deserializing JSON message: " & ex.ToString)
                        End Try

                        '#End Region ' Process HTTP message as JSON
                    ElseIf request_two.Headers("Content-Type").Contains("x-www-form-urlencoded") Then
                        Trace.WriteLineIf(_traceSwitch.TraceVerbose, "Received json content from client.")
                        _messageFormat = MessageFormat.json

                        '#Region "Process HTTP x-www-form-urlencoded message as JSON"

                        Try
                            Dim jsonSerializer As New JavaScriptSerializer
                            Dim clientMessage As ClientMessage3 = jsonSerializer.Deserialize(Of ClientMessage3)(request_two.Body)
                            _lastUpdate = clientMessage.SendTimestamp
                            Trace.WriteLineIf(_traceSwitch.TraceVerbose, "Received the following x-www-form-urlencoded message: ")
                            Trace.WriteLineIf(_traceSwitch.TraceVerbose, "SendTimestamp: " & clientMessage.SendTimestamp.ToString)
                            Trace.WriteLineIf(_traceSwitch.TraceVerbose, "Browser: " & clientMessage.Browser)
                            Trace.WriteLineIf(_traceSwitch.TraceVerbose, "Message: " & clientMessage.Message)
                        Catch ex As Exception
                            Trace.WriteLineIf(_traceSwitch.TraceVerbose, "Error deserializing JSON message: " & ex.ToString)
                        End Try

                        '#End Region ' Process HTTP x-www-form-urlencoded message as JSON
                    ElseIf request_two.Headers("Content-Type").Contains("ping") Then
                        Dim msg As String = request_two.Body
                        Trace.WriteLineIf(_traceSwitch.TraceVerbose, "Ping received.")
                        If msg.Equals("PING") Then
                            Dim args As New SendMessageEventArgs
                            args.Client = Me
                            args.Message = "PING"
                            args.Request = request_two
                            args.Timestamp = Date.Now
                            SendResponse(args)
                        End If
                    Else
                        Trace.WriteLineIf(_traceSwitch.TraceVerbose, "Received " & request_two.Headers("Content-Type") & " content from client.")
                        _messageFormat = MessageFormat.querystring
                        Dim msgString As String = request_two.Body
                        Trace.WriteLineIf(_traceSwitch.TraceVerbose, "Message from client: " & msgString)
                        Dim separators(0) As Char
                        If msgString.Contains(",") Then
                            separators(0) = ","c
                        Else
                            separators(0) = "&"c
                        End If
                        Dim keyValuePairs() As String = msgString.Split(separators)

                        For Each keyValuePair As String In keyValuePairs
                            Dim keyName As String
                            Dim valueString As String
                            If keyValuePair.Contains("=") Then
                                keyName = keyValuePair.Substring(0, keyValuePair.IndexOf("="c))
                                valueString = keyValuePair.Substring(keyValuePair.IndexOf("="c) + 1)
                            Else
                                keyName = keyValuePair.Substring(0, keyValuePair.IndexOf(":"c))
                                valueString = keyValuePair.Substring(keyValuePair.IndexOf(":"c) + 1)
                                keyName = keyName.Substring(1, keyName.Length - 1)
                                valueString = valueString.Substring(1, valueString.Length - 1)
                            End If
                            Trace.WriteLineIf(_traceSwitch.TraceVerbose, "Key = " & keyName & ", Value = " & valueString)

                            If keyName.Equals("callback") Then
                                _callbackValue = valueString
                                Continue For
                            End If

                            _nameValuePairs.Add(keyName, valueString)
                        Next keyValuePair
                    End If
                End If
            Catch ex As Exception
                Trace.WriteLineIf(_traceSwitch.TraceVerbose, "Client.ParseRequestContent general exception: " & ex.ToString)
            End Try
        End Sub

        Private Sub ParseInput()
            Dim pos As Integer
            Dim temp As String
            Dim skip As Boolean = False

            Trace.WriteLineIf(_traceSwitch.TraceVerbose, "Inside ParseInput...")

            Do While bufferpos < buffer.Length - 1 AndAlso state <> ParseState.Done AndAlso Not skip
                Select Case state
                    Case ParseState.Start
                        Trace.WriteLineIf(_traceSwitch.TraceVerbose, "Inside ParseInput(): ParseState.Start")
                        pos = buffer.IndexOf(vbCrLf, bufferpos)
                        If pos >= 0 Then
                            temp = buffer.Substring(bufferpos, pos - bufferpos)
                            bufferpos = pos + 2

                            Dim parts() As String = temp.Split(" "c)
                            If parts.Length = 3 Then
                                request_two = New Request

                                request_two.Method = parts(0)
                                request_two.Url = parts(1)
                                Trace.WriteLineIf(_traceSwitch.TraceVerbose, "Request Url: " & request_two.Url)
                                request_two.Version = parts(2)

                                state = ParseState.Headers
                            Else
                                CleanUp()
                            End If
                        End If
                    Case ParseState.Headers
                        Trace.WriteLineIf(_traceSwitch.TraceVerbose, "Inside ParseInput(): ParseState.Headers")
                        pos = buffer.IndexOf(vbCrLf, bufferpos)
                        If pos >= 0 Then
                            temp = buffer.Substring(bufferpos, pos - bufferpos)
                            bufferpos = pos + 2

                            If temp.Length > 0 Then
                                Dim parts() As String = temp.Split(New String(0) {": "}, StringSplitOptions.RemoveEmptyEntries)
                                If parts.Length = 2 Then
                                    request_two.Headers.Add(parts(0), parts(1))
                                    Trace.WriteLineIf(_traceSwitch.TraceVerbose, "Parsing header: " & parts(0) & " : " & parts(1))
                                End If
                            Else
                                If request_two.HasContent Then
                                    state = ParseState.Content
                                Else
                                    state = ParseState.Done
                                End If
                            End If
                        End If
                    Case ParseState.Content
                        Trace.WriteLineIf(_traceSwitch.TraceVerbose, "Inside ParseInput(): ParseState.Content")
                        If bufferpos + request_two.ContentLength <= buffer.Length Then
                            Dim t As Integer = bufferpos + request_two.ContentLength - 1
                            request_two.Body = buffer.Substring(bufferpos, request_two.ContentLength)
                            bufferpos += request_two.ContentLength

                            state = ParseState.Done
                        Else
                            skip = True
                        End If
                End Select
            Loop

            If state = ParseState.Done Then
                Trace.WriteLineIf(_traceSwitch.TraceVerbose, "Inside ParseInput(): ParseState.Done")
                state = ParseState.Start
                ParseRequestContent()

                If request_two.VerifyAuth(authconfig) Then
                    Trace.WriteLineIf(_traceSwitch.TraceVerbose, "This client has been authenticated.")
                    _authenticated = True
                Else
                    Trace.WriteLineIf(_traceSwitch.TraceVerbose, "This client failed authentication.")
                    Dim response As Response = Response.Get401Response(authconfig)
                    response.SendResponse(stream, Me)
                End If
            End If

            Trace.WriteLineIf(_traceSwitch.TraceVerbose, "Leaving Client.ParseInput.")
        End Sub
	End Class
End Namespace

