Imports System.Threading
Imports System.Text
Imports System.Net
Imports System.IO
Imports System.Collections
Imports System.Net.Sockets
Imports System.Configuration

Namespace Derrick.Web
	Public Class Response
		Public Status As Status = Status.OK
		Public Mime As String = "text/html"
        'Public Mime As String = "application/octet-stream"
		Public Body As String = ""
		Public File As FileInfo = Nothing
		Private Headers As New Dictionary(Of String, String)

		Public Property LastUpdate As Date

        'Public Property SendFlag As Boolean
		Private _traceSwitch As TraceSwitch

		Public Sub SendResponse(ByVal stream As NetworkStream, ByVal client As Client)
			_traceSwitch = New TraceSwitch("TraceLevelSwitch", "COMET Client.")

			If File IsNot Nothing AndAlso (Not File.Exists) Then
				Exit Sub
			End If

			Try
				'add cross domain headers:
                'AddHeader("Access-Control-Allow-Origin", client.Request.Url)
                'AddHeader("Access-Control-Allow-Origin", "*")

                'Dim fullUrl As String = client.Request.Url
                'Dim accessAllowedUrl As String = fullUrl.Substring(0, fullUrl.IndexOf("/"))
				If client.Request.Headers.ContainsKey("Origin") Then
					AddHeader("Access-Control-Allow-Origin", client.Request.Headers("Origin"))
					Trace.WriteLineIf(_traceSwitch.TraceVerbose, "Access-Control-Allow-Origin from client: " & client.Request.Headers("Origin"))
				Else
					AddHeader("Access-Control-Allow-Origin", ConfigurationManager.AppSettings("RequestOriginUrl"))
					Trace.WriteLineIf(_traceSwitch.TraceVerbose, "Access-Control-Allow-Origin from config: " & ConfigurationManager.AppSettings("RequestOriginUrl"))
				End If

				AddHeader("Access-Control-Allow-Methods", "POST, GET, OPTIONS")
				AddHeader("Access-Control-Max-Age", "1000")
                'AddHeader("Access-Control-Allow-Headers", "Content-Type")
				Dim allowHeaders As String = ConfigurationManager.AppSettings("AllowHeaders")
                'AddHeader("Access-Control-Allow-Headers", "Content-Type, x-requested-with")
				AddHeader("Access-Control-Allow-Headers", allowHeaders)

				Dim r As New StringBuilder

				r.Append("HTTP/1.1 " & GetStatusString(Status) & vbCrLf)
				r.Append("Server: Derrick Comet" & vbCrLf)
				r.Append("Date: " & Date.Now.ToUniversalTime.ToString("ddd, dd MMM yyyy HH':'mm':'ss 'GMT'") & vbCrLf)
				r.Append("Accept-Ranges: none" & vbCrLf)

				For Each header As KeyValuePair(Of String, String) In Headers
					r.Append(header.Key & ": " & header.Value & vbCrLf)
				Next header

				If File IsNot Nothing Then
					r.Append("Content-Type: " & Mime & vbCrLf)
					r.Append("Content-Length: " & File.Length & vbCrLf)
				ElseIf Body.Length > 0 Then
					r.Append("Content-Type: " & Mime & vbCrLf)
					r.Append("Content-Length: " & Body.Length & vbCrLf)
				End If
				r.Append(vbCrLf)

                'Console.WriteLine(r.ToString)

				Dim htext() As Byte = Encoding.ASCII.GetBytes(r.ToString)
				stream.Write(htext, 0, htext.Length)

				If Body <> "" Then
					'add 2KB of space
                    'For count = 0 To 2047

                    'Body = " " & Body

                    'Next count

					client.Send(Body)
				ElseIf File IsNot Nothing Then
					Using fs As FileStream = File.OpenRead
						Dim b(1023) As Byte
						Dim bytesRead As Integer
						bytesRead = fs.Read(b, 0, 1024)
						Do While bytesRead > 0
							stream.Write(b, 0, bytesRead)
							bytesRead = fs.Read(b, 0, 1024)
						Loop
						fs.Close()
					End Using
				End If
			Catch ex As Exception
				Trace.WriteLine("Exception thrown when sending response: " & ex.ToString)
				Trace.Flush()
			End Try
		End Sub

		Public Sub AddHeader(ByVal key As String, ByVal value As String)
			Headers.Add(key, value)
		End Sub

		Private Function GetStatusString(ByVal status As Status) As String
			Select Case status
				Case Status.OK
					Return "200 OK"
				Case Status.NotAuthorized
					Return "401 Not Authorized"
				Case Status.NotFound
					Return "404 Not Found"
				Case Status.InternalServerError
					Return "500 Internal Server Error"
				Case Else
					Throw New Exception("Invalid HTTP response.")
			End Select
		End Function

		Public Shared Function Get404Response(ByVal filename As String) As Response
			Dim res As New Response
			res.Status = Status.NotFound
			res.Mime = "text/html"
			res.Body = "<html>" & ControlChars.CrLf & "<head>" & ControlChars.CrLf & "    <title>404 Not found.</title>" & ControlChars.CrLf & "</head>" & ControlChars.CrLf & "<body>" & ControlChars.CrLf & "    404: The file '" & filename & "' cannot be found." & ControlChars.CrLf & "</body>" & ControlChars.CrLf & "</html>"
			Return res
		End Function

		Public Shared Function Get500Response(ByVal errormsg As String) As Response
			Dim res As New Response
			res.Status = Status.InternalServerError
			res.Mime = "text/html"
			res.Body = "<html>" & ControlChars.CrLf & "<head>" & ControlChars.CrLf & "    <title>500 Internal server error.</title>" & ControlChars.CrLf & "</head>" & ControlChars.CrLf & "<body>" & ControlChars.CrLf & "    500: There was an internal server error with the message: " & errormsg & " " & ControlChars.CrLf & "</body>" & ControlChars.CrLf & "</html>"
			Return res
		End Function

		Public Shared Function Get401Response(ByVal authconfig As AuthConfig) As Response
			Dim res As New Response
			res.Status = Status.NotAuthorized
			res.Mime = "text/html"
			res.Body = "<html>" & ControlChars.CrLf & "<head>" & ControlChars.CrLf & "    <title>401 Not authorized.</title>" & ControlChars.CrLf & "</head>" & ControlChars.CrLf & "<body>" & ControlChars.CrLf & "    401: You are not authorized to view this resource." & ControlChars.CrLf & "</body>" & ControlChars.CrLf & "</html>"
			res.AddHeader("WWW-Authenticate", authconfig.Type.ToString & " realm=""" & authconfig.Realm & """")
			Return res
		End Function

		Public Shared Function GetFileResponse(ByVal file As FileInfo, ByVal Url As String) As Response
			If file.Exists Then
				Try
'                    
                    '                    Dim data As String = ""
                    '                    Using s As FileStream = file.OpenRead
                    '                        s.Read(bytes, 0, CInt(Fix(s.Length)))
                    '                                    data = Encoding.ASCII.GetString(bytes, 0, bytesread)
                    '                       Byte) bytes = New Byte(s.Length
                    'Integer bytesread = s.Read(bytes, 0, CInt(Fix(s.Length)))
                    '                        data = Encoding.ASCII.GetString(bytes, 0, bytesread)


					Dim res As New Response
					res.File = file
					res.Mime = GetMimeType(file.Extension)
					Return res
				Catch e As Exception
                    Return Get500Response("Error reading file: " & e.ToString)
				End Try
			Else
				Return Get404Response(file.Name & " (" & Url & ")")
			End If
		End Function

		Public Shared Function GetHtmlResponse(ByVal html As String, ByVal lastUpdate As Date, ByVal messageFormat As MessageFormat) As Response
			Dim res As New Response
			res.Body = html
			res.Status = Status.OK
			If messageFormat = MessageFormat.json Then
				res.Mime = "text/x-json"
			ElseIf messageFormat = MessageFormat.xml Then
				res.Mime = "text/xml"
			Else
				res.Mime = "text/html"
			End If
            'res.Mime = "application/octet-stream"
			res.LastUpdate = lastUpdate
			Return res
		End Function

		Public Shared Function GetMimeType(ByVal e As String) As String
			If e = ".jpg" Then
				Return "image/jpeg"
			ElseIf e = ".gif" Then
				Return "image/gif"
			Else
				Return "text/html"
			End If
		End Function
	End Class
End Namespace
