Imports System.ComponentModel
Imports System.ServiceProcess
Imports System.Text
Imports System.Configuration
Imports System.Threading
Imports System.IO

Namespace Derrick.Web
	Partial Public Class CometService
		Inherits ServiceBase

		Public Shared Configuration As AppConfig

		Private Shared _wiserver As WIServer
		Private Shared _siserver As SIServer
		Private Shared _messageQueueHandler As MessageLoop

		Private _traceSwitch As TraceSwitch

		Public Sub New()
			InitializeComponent()
		End Sub

		Protected Overrides Sub OnStart(ByVal args() As String)
			GC.Collect()

			Dim configfile As String = ""

			'Set up tracing:
			_traceSwitch = New TraceSwitch("TraceLevelSwitch", "Tracing level of COMET server.")

			Trace.WriteLineIf(_traceSwitch.TraceVerbose, "Derrick COMET service OnStart() called...")

			Dim cometSleepTimeStr As String = ConfigurationManager.AppSettings("CometSleepTime")
			Dim cometSleepTime As Integer = 5000
			If Not String.IsNullOrEmpty(cometSleepTimeStr) Then
				cometSleepTime = Int32.Parse(cometSleepTimeStr)
			End If
			Thread.Sleep(cometSleepTime)

			Try
				If args.Length = 0 Then
					configfile = ConfigurationManager.AppSettings("CometConfig")
					If String.IsNullOrEmpty(configfile) Then
						configfile = "cometbox.conf"
					End If
				Else
					configfile = args(0)
				End If

				If Not New FileInfo(configfile).Exists Then
					Try
						AppConfig.BuildNewConfigFile(configfile)
					Catch ex As Exception
						Trace.WriteLineIf(_traceSwitch.TraceError, ex.ToString, "CometService")
					End Try
				End If

				If New FileInfo(configfile).Exists Then
					Configuration = AppConfig.LoadConfig(configfile)
					If Configuration Is Nothing Then
                        'Console.WriteLine("Error loading configuration.")
						GracefullyChokeAndDie()
						Exit Sub
					End If
				Else
                    'Console.WriteLine("Configuration not found: " & configfile)
					GracefullyChokeAndDie()
					Exit Sub
				End If

				_wiserver = New WIServer(Configuration.WebInterface)
				_siserver = New SIServer(Configuration.ServerInterface)
				_messageQueueHandler = New MessageLoop
				AddHandler _messageQueueHandler.SendMessageEvent, AddressOf _siserver.BroadcastMessage
				AddHandler _messageQueueHandler.SendMessageEvent, AddressOf _wiserver.BroadcastMessage

				Trace.WriteLineIf(_traceSwitch.TraceVerbose, "Derrick COMET service OnStart() completed.")
			Catch ex As Exception
				Trace.WriteLineIf(_traceSwitch.TraceError, ex.ToString)
			End Try
		End Sub

		Protected Overrides Sub OnStop()
            '_wiserver.CloseListener
            '_siserver.CloseListener

			_wiserver.ShutDown()
			_siserver.ShutDown()
            GracefullyChokeAndDie()

		End Sub

		Private Shared Sub GracefullyChokeAndDie()
            'Console.WriteLine("")
            'Console.WriteLine("Press any key to exit...")		
            'Console.ReadKey(true)
			GC.Collect()
			Trace.WriteLine("Gracefully exiting...")
			Trace.Flush()
			Trace.Close()
		End Sub
	End Class
End Namespace
