Imports System.Threading
Imports System.Text
Imports System.Net
Imports System.Net.Sockets
Imports System.Collections

Namespace Derrick.Web
	Public MustInherit Class Server
		Protected _listenerThread As Thread = Nothing
		Private listener As TcpListener = Nothing
		Private authconfig As AuthConfig
		Protected _traceSwitch As TraceSwitch
		Protected clients As New List(Of Client)
        'ORIGINAL LINE: Protected _lockObj As Object = New Object
		Protected _lockObj As New Object
		Protected _isListenerShutDown As Boolean = False
		Protected Shared _listenerMutex As New Mutex

		Private _currentClientId As Integer = 0

		Public Property IsListenerShutDown As Boolean
			Get
				Return _isListenerShutDown
			End Get
			Set(ByVal value As Boolean)
				_isListenerShutDown = value
			End Set
		End Property

		Public ReadOnly Property ListenerMutex As Mutex
			Get
				Return _listenerMutex
			End Get
		End Property

		Public Sub New(ByVal ip As IPAddress, ByVal port As Int32, ByVal a As AuthConfig)
			Try
				_traceSwitch = New TraceSwitch("TraceLevelSwitch", "Comet Tracing")

				authconfig = a

                'listener = new TcpListener(port)
				listener = New TcpListener(ip, port)
				listener.ExclusiveAddressUse = True
				listener.Start()

                _listenerThread = New Thread(New ThreadStart(AddressOf nLoop))
				_listenerThread.Start()
			Catch ex As Exception
				EventLog.WriteEntry("Derrick COMET", ex.ToString, EventLogEntryType.Error)
				Trace.WriteLineIf(_traceSwitch.TraceError, "Error starting on port " & port & ": " & ex.ToString, "COMET Server")
			End Try
		End Sub

        Private Sub nLoop()
            Try
                'Dim clients As New List(Of Client)

                Do
                    Dim client As TcpClient = Nothing

                    Dim isServerStopped As Boolean = False
                    _listenerMutex.WaitOne()
                    isServerStopped = _isListenerShutDown
                    _listenerMutex.ReleaseMutex()

                    If Not isServerStopped Then
                        client = listener.AcceptTcpClient
                    Else
                        Continue Do
                    End If

                    Trace.WriteLineIf(_traceSwitch.TraceInfo, "TCP client accepted.", "COMET Server")

                    Dim addClientFlag As Boolean = True

                    Dim dc As New Client(client, Me, authconfig, _currentClientId)
                    _currentClientId += 1
                    For Each currentClient As Client In clients
                        If dc.TCPClient Is currentClient.TCPClient Then
                            SyncLock _lockObj
                                addClientFlag = False
                            End SyncLock
                        End If
                    Next currentClient

                    If addClientFlag Then
                        SyncLock _lockObj
                            clients.Add(dc)
                        End SyncLock
                    End If

                    Trace.WriteLineIf(_traceSwitch.TraceInfo, "Number of TCP clients: " & clients.Count)

                    Dim i As Integer = 0
                    Do While i < clients.Count
                        If Not clients(i).IsLive Then
                            clients.RemoveAt(i)
                        End If
                        i += 1
                    Loop
                Loop
            Catch ex As Exception
                'EventLog.WriteEntry("Derrick COMET", ex.ToString, EventLogEntryType.Error)
                Trace.WriteLineIf(_traceSwitch.TraceError, ex.ToString, "COMET Server")
            End Try
        End Sub

		Public Function IsRunning() As Boolean
			Return _listenerThread.IsAlive
		End Function

		Public Sub CloseListener()
			If listener IsNot Nothing Then
				_listenerMutex.WaitOne()
				_isListenerShutDown = True
				listener.Stop()
				_listenerMutex.ReleaseMutex()
			End If
		End Sub

		Public Sub ShutDown()
			CloseListener()
			For Each c As Client In clients
				If c IsNot Nothing Then
					c.CleanUp()
				End If
			Next c
		End Sub

		Public MustOverride Function HandleRequest(ByVal request As Request, ByVal lastUpdate As Date, ByVal cometClient As Client) As Response

		Public MustOverride Sub BroadcastMessage(ByVal sender As Object, ByVal args As SendMessageEventArgs)
	End Class
End Namespace
