Imports System.Messaging
Imports System.Configuration
Imports System.Threading
Imports System.Data.OleDb
Imports Derrick.Web

Namespace Derrick.Web
	Public Class CometMessage
		Public Property Message As String
		Public SentToClients As New List(Of Integer)

		Public Property LastUpdated As Date
	End Class

	Public NotInheritable Class QueueMessage
        'Public Property LastUpdated As Date
        'Public Property Message As String

		#Region "Singleton specific"
        'ORIGINAL LINE: Private Shared _instance As QueueMessage
		Private Shared _instance As QueueMessage
		Private Shared _syncRoot As New Object
		Public Shared ReadOnly Property Instance As QueueMessage
			Get
				If Nothing Is _instance Then
					SyncLock _syncRoot
						If _instance Is Nothing Then
							_instance = New QueueMessage
						End If
					End SyncLock
				End If
				Return _instance
			End Get
		End Property
		#End Region ' Singleton specific

		Private Shared _lastUpdated As Date
		Private Shared _message As String

		

		Public Property LastUpdated As Date
			Get
				Return _lastUpdated
			End Get
			Set(ByVal value As Date)
				_lastUpdated = value
			End Set
		End Property

		Public Property Message As String
			Get
				Return _message
			End Get
			Set(ByVal value As String)
				_message = value
			End Set
		End Property

		Public Messages As New List(Of CometMessage)
	End Class

	Public Class SendMessageEventArgs
		Inherits EventArgs

		Public Property Request As Request

		Public Property Timestamp As Date

		Public Property Client As Client

		Public Property Message As String
	End Class

	Public Class MessageLoop
		Protected _intranetBannerQueue As MessageQueue
		Protected _messageHandlingThread As Thread = Nothing
		Protected _traceSwitch As TraceSwitch

		'Send message event
		Public Delegate Sub SendMessageEventHandler(ByVal sender As Object, ByVal args As SendMessageEventArgs)
		Public Event SendMessageEvent As SendMessageEventHandler

		Public Sub New()
			_traceSwitch = New TraceSwitch("TraceLevelSwitch", "Comet Tracing")

			Dim msgQueueName As String = ConfigurationManager.AppSettings("IntranetBannerQueue")
			Trace.WriteLineIf(_traceSwitch.TraceInfo, "Intranet queue name: " & msgQueueName)
			_intranetBannerQueue = New MessageQueue(msgQueueName)
			Trace.WriteLineIf(_traceSwitch.TraceInfo, "New message queue instance created.")

			_messageHandlingThread = New Thread(New ThreadStart(AddressOf messageLoop_Renamed))
			_messageHandlingThread.Start()
		End Sub

		Private Sub messageLoop_Renamed()
			Try
				Do
					Trace.WriteLineIf(_traceSwitch.TraceInfo, "Awaiting message from the message queue.")
					Dim msg As Message = _intranetBannerQueue.Receive 'holds thread until message received
					Trace.WriteLineIf(_traceSwitch.TraceInfo, "Message retrieved from the message queue.")

					Dim args As New SendMessageEventArgs
					args.Timestamp = Date.Now.ToUniversalTime

					msg.Formatter = New XmlMessageFormatter(New Type() { GetType(String) })
					Dim cometMsg As String = msg.Body.ToString
					args.Message = cometMsg

                    RaiseEvent SendMessageEvent(Me, args)
                    Trace.WriteLineIf(_traceSwitch.TraceVerbose, "Message loop raised SendMessage event.")

                    Trace.WriteLineIf(_traceSwitch.TraceInfo, "Message is: " & cometMsg)
                Loop
			Catch ex As Exception
				Trace.WriteLineIf(_traceSwitch.TraceError, ex.ToString, "COMET Server")
			End Try
		End Sub
	End Class
End Namespace
