Imports System.Text
Imports System.Threading
Imports System.Net
Imports System.IO
Imports System.Configuration
Imports System.Messaging
Imports System.Web.Script.Serialization

Namespace CometTester
	Public Class TestResultItem
		Public TimeTakenInMilliseconds As Double
		Public RepsFailed As Integer
		Public WcfFailures As Integer
	End Class

	Public Class CometConnectionEventArgs
		Inherits EventArgs
	End Class

	Public Class CometTester
		Private _id As Integer
		Private _doneEvent As ManualResetEvent
		Private _message As String
		Private _repsPerTest As Integer
		Private _result As New TestResultItem
		Private _waitTime As Integer = 0

		Public ReadOnly Property ID As Integer
			Get
				Return _id
			End Get
		End Property

		Public ReadOnly Property TestResults As TestResultItem
			Get
				Return _result
			End Get
		End Property

		Public Event CometConnectionEvent As EventHandler(Of CometConnectionEventArgs)

		Public Sub New(ByVal id As Integer, ByVal doneEvent As ManualResetEvent, ByVal message As String, ByVal repsPerTest As Integer, ByVal waitTime As Integer)
			_id = id
			_doneEvent = doneEvent
			_message = message
			_repsPerTest = repsPerTest
			_result.RepsFailed = 0
			_result.WcfFailures = 0
			_waitTime = waitTime
		End Sub

		#Region "Synchronous Test Methods"
		Public Sub SingleCometRoundtrip()
			Thread.Sleep(1000)

			Dim request As HttpWebRequest = CType(HttpWebRequest.Create(ConfigurationManager.AppSettings("CometServerUrl")), HttpWebRequest)

			Dim bytes() As Byte
			Using reader As Stream = File.OpenRead("Data.xml")
                bytes = New Byte(CInt(reader.Length - 1)) {}
				reader.Read(bytes, 0, CInt(Fix(reader.Length)))
			End Using

			request.SendChunked = False
			request.KeepAlive = True
			request.Credentials = New NetworkCredential("server", "server123")
			request.Method = "GET"
            'request.ContentType = "application/xml"
			request.ContentType = "text/json"
			request.ContentLength = bytes.Length - 3

			Dim stream As Stream = request.GetRequestStream
			stream.Write(bytes, 3, bytes.Length - 3)

            'Console.WriteLine(System.Text.Encoding.ASCII.GetString(bytes))

			Dim messageReturned As String

			Try
				Dim res As HttpWebResponse = CType(request.GetResponse, HttpWebResponse)
				Dim s As New StreamReader(res.GetResponseStream)
				messageReturned = s.ReadToEnd
				s.Close()
			Catch e As Exception
				Trace.WriteLine(e.ToString)
			End Try
			stream.Close()
		End Sub

		Public Sub EngageCometBroadcast(ByVal threadContext As Object)
			Dim threadIndex As Integer = CInt(Fix(threadContext))

            'Thread.Sleep(1000)

			Dim messageReturned As String
			Dim s As StreamReader
			Dim testPassed As Boolean = True

			For i  = 0 To _repsPerTest - 1
				Try
					Dim request As HttpWebRequest = CType(HttpWebRequest.Create(ConfigurationManager.AppSettings("CometServerUrl")), HttpWebRequest)

					Dim bytes() As Byte
					Dim encoding As New UTF8Encoding
					bytes = encoding.GetBytes(_message)

					request.SendChunked = False
					request.KeepAlive = True
					request.Credentials = New NetworkCredential("server", "server123")
					request.Method = "POST"
                    'request.ContentType = "application/xml"
					request.ContentType = "text/json"
					request.ContentLength = bytes.Length
					request.KeepAlive = True

					Dim stream As Stream = Nothing

					stream = request.GetRequestStream
					stream.Write(bytes, 0, bytes.Length)
					Try
                        'Dim callCometClient As MsgQService.CallCometServiceClient = New MsgQService.CallCometServiceClient
                        'callCometClient.SendCometMessage(_message)
                        'callCometClient.Close
					Catch ex As Exception
						_result.WcfFailures += 1
						Trace.WriteLine(ex.ToString)
						Continue For
					End Try
					Thread.Sleep(_waitTime)
					Dim startTime As Date = Date.Now
					Dim res As HttpWebResponse = CType(request.GetResponse, HttpWebResponse)
					s = New StreamReader(res.GetResponseStream)
					messageReturned = s.ReadToEnd
					s.Close()
					res.Close()
					Dim endTime As Date = Date.Now
					Dim singleCometResponse As Double = endTime.Subtract(startTime).TotalMilliseconds
					_result.TimeTakenInMilliseconds += endTime.Subtract(startTime).TotalMilliseconds
					If String.IsNullOrEmpty(messageReturned) Then
						'failed.
						_result.RepsFailed += 1
					End If
					stream.Close()
				Catch ex As Exception
					testPassed = False
					Trace.WriteLine(ex.ToString)
					_result.RepsFailed += 1
				End Try
			Next i

			_result.TimeTakenInMilliseconds = _result.TimeTakenInMilliseconds / _repsPerTest

			_doneEvent.Set()
		End Sub

		Public Sub PingComet(ByVal threadContext As Object)
			Dim threadIndex As Integer = CInt(Fix(threadContext))

            'Thread.Sleep(1000)

			Dim messageReturned As String
			Dim s As StreamReader
			Dim testPassed As Boolean = True

			For i  = 0 To _repsPerTest - 1
				Try
					Dim request As HttpWebRequest = CType(HttpWebRequest.Create(ConfigurationManager.AppSettings("CometServerUrl")), HttpWebRequest)

					Dim bytes() As Byte
					Dim encoding As New UTF8Encoding
					bytes = encoding.GetBytes(_message)

					request.SendChunked = False
					request.KeepAlive = True
					request.Credentials = New NetworkCredential("server", "server123")
					request.Method = "POST"
                    'request.ContentType = "application/xml"
					request.ContentType = "text/ping"
					request.ContentLength = bytes.Length
					request.KeepAlive = True

					Dim stream As Stream = Nothing

					stream = request.GetRequestStream
					stream.Write(bytes, 0, bytes.Length)
					Thread.Sleep(_waitTime)
					Dim startTime As Date = Date.Now
					Dim res As HttpWebResponse = CType(request.GetResponse, HttpWebResponse)
					Dim endTime As Date = Date.Now
					s = New StreamReader(res.GetResponseStream)
					messageReturned = s.ReadToEnd
					s.Close()
					res.Close()
					Dim singleCometResponse As Double = endTime.Subtract(startTime).TotalMilliseconds
					_result.TimeTakenInMilliseconds += endTime.Subtract(startTime).TotalMilliseconds
					If String.IsNullOrEmpty(messageReturned) Then
						'failed.
						_result.RepsFailed += 1
					End If
					stream.Close()
				Catch ex As Exception
					testPassed = False
					Trace.WriteLine(ex.ToString)
					_result.RepsFailed += 1
				End Try
			Next i

			_result.TimeTakenInMilliseconds = _result.TimeTakenInMilliseconds / _repsPerTest

			_doneEvent.Set()
		End Sub
		#End Region ' Synchronous Test Methods

		#Region "Asynchronous Test Methods"
		Private _asyncCallDone As New ManualResetEvent(False)

		Public Sub AsyncPing(ByVal threadContext As Object)
			Dim threadIndex As Integer = CInt(Fix(threadContext))

			Dim cometSite As New Uri(ConfigurationManager.AppSettings("CometServerUrl"))
			Dim wreq As WebRequest = WebRequest.Create(cometSite)
			Dim rs As New RequestState
			Dim bytes() As Byte
			Dim encoding As New UTF8Encoding
			bytes = encoding.GetBytes(_message)

			wreq.Credentials = New NetworkCredential("server", "server123")
			wreq.Method = "POST"
            'request.ContentType = "application/xml"
			wreq.ContentType = "text/json"
			wreq.ContentLength = bytes.Length

			rs.Request = wreq
			rs.StartTime = Date.Now
			Dim r As IAsyncResult = CType(wreq.BeginGetResponse(New AsyncCallback(AddressOf asyncPingCallback), rs), IAsyncResult)
            '_asyncCallDone.WaitOne
			_doneEvent.Set()
		End Sub

		Private Sub asyncPingCallback(ByVal ar As IAsyncResult)
			Dim rs As RequestState = CType(ar.AsyncState, RequestState)
			Dim req As WebRequest = rs.Request
			Dim resp As WebResponse = req.EndGetResponse(ar)
			Dim endTime As Date = Date.Now
			_result.TimeTakenInMilliseconds = endTime.Subtract(rs.StartTime).TotalMilliseconds
            'Stream responseStream = resp.GetResponseStream
		End Sub

		'Async call that includes transmitting message via message queue:

		Public Sub AsyncTest(ByVal threadContext As Object)
			Dim threadIndex As Integer = CInt(Fix(threadContext))

			Dim cometSite As New Uri(ConfigurationManager.AppSettings("CometServerUrl"))
			Dim wreq As WebRequest = WebRequest.Create(cometSite)
			Dim rs As New RequestState
			Dim bytes() As Byte
			Dim encoding As New UTF8Encoding
			bytes = encoding.GetBytes("Open sesame.")

			wreq.Credentials = New NetworkCredential("server", "server123")
			wreq.Method = "POST"
            'request.ContentType = "application/xml"
			wreq.ContentType = "text/json"
			wreq.ContentLength = bytes.Length
			Dim requestStream As Stream = wreq.GetRequestStream
			requestStream.Write(bytes, 0, bytes.Length)

			rs.Request = wreq
			rs.StartTime = Date.Now

            'Dim handler As EventHandler(Of CometConnectionEventArgs) = CometConnectionEvent

            'If handler IsNot Nothing Then

            'handler(Me, New CometConnectionEventArgs)

            ' End If

            Dim r As IAsyncResult = CType(wreq.BeginGetResponse(New AsyncCallback(AddressOf asyncTestCallback), rs), IAsyncResult)

            '_asyncCallDone.WaitOne
            '_doneEvent.Set
		End Sub

		Private Sub asyncTestCallback(ByVal ar As IAsyncResult)
			Dim rs As RequestState = CType(ar.AsyncState, RequestState)
			Dim req As WebRequest = rs.Request
			Dim resp As WebResponse = Nothing
			Try
				resp = req.EndGetResponse(ar)
			Catch ex As Exception
				_result.RepsFailed += 1
				_doneEvent.Set()
				Exit Sub
			End Try
			Dim endTime As Date = Date.Now.ToUniversalTime
			Dim responseMessageStream As Stream = resp.GetResponseStream

			Dim jsonSerializer As JavaScriptSerializer = Nothing

			Dim responseMessageReader As New StreamReader(responseMessageStream)
			Dim responseMessageStr As String = responseMessageReader.ReadToEnd

			Try
				jsonSerializer = New JavaScriptSerializer
			Catch ex As Exception
				_result.RepsFailed += 1
				_doneEvent.Set()
				Exit Sub
			End Try
            '_result.TimeTakenInMilliseconds = endTime.Subtract(rs.StartTime).TotalMilliseconds
            'Stream responseStream = resp.GetResponseStream

			Dim responseMessage As ClientMessage3 = jsonSerializer.Deserialize(Of ClientMessage3)(responseMessageStr)
			_result.TimeTakenInMilliseconds += endTime.Subtract(responseMessage.SendTimestamp).TotalMilliseconds

            '_asyncCallDone.Set
			_doneEvent.Set()
		End Sub

		Public Sub DumpMessage(ByVal threadContext As Object)
			Try
                '_cometServer.BroadcastMessage(_sendCometMsgTextBox.Text)
				Dim msgQueueName As String = ConfigurationManager.AppSettings("CometQueue")
				Dim testMessageQueue As New MessageQueue(msgQueueName)
				Dim msg As New Messaging.Message(_message, New XmlMessageFormatter(New Type() { GetType(String) }))
				testMessageQueue.Send(msg)
			Catch ex As Exception
                Dim err As String = ex.ToString
                'Response.Write(error)
				Throw ex
			End Try
		End Sub
		#End Region ' Asynchronous Test Methods

	End Class

	Public Class RequestState
		Private Const BufferSize As Integer = 1024
		Public RequestData As StringBuilder
		Public BufferRead() As Byte
		Public Request As WebRequest
		Public ResponseStream As Stream
		' Create Decoder for appropriate enconding type.
		Public StreamDecode As Decoder = Encoding.UTF8.GetDecoder
		Public StartTime As Date

		Public Sub New()
			BufferRead = New Byte(BufferSize - 1){}
			RequestData = New StringBuilder(String.Empty)
			Request = Nothing
			ResponseStream = Nothing
		End Sub
	End Class

	Public Class ClientMessage3
		Public Property SendTimestamp As Date

		Public Property Browser As String

		Public Property Message As String
	End Class
End Namespace
