Imports System.ComponentModel
Imports System.Text
Imports System.Threading
Imports System.Configuration
Imports System.Messaging

Namespace CometTester
	Partial Public Class MainForm
		Inherits Form

		Private _numOfConnections As Integer = 0
		Private _totalUsers As Integer = 0
		Private _message As String

		Public Sub New()
			InitializeComponent()
		End Sub

		Private Sub _executeLoadTestButton_Click(ByVal sender As Object, ByVal e As EventArgs) Handles _executeLoadTestButton.Click
			Dim maxUsers As Integer = Convert.ToInt32(_maxUserLoad.Text)
			Dim repsPerSet As Integer = Convert.ToInt32(_repsPerLoadSet.Text)
			Dim bytesPerMsg As Integer = Convert.ToInt32(_bytesPerMessage.Text)
			Dim waitTime As Integer = Convert.ToInt32(_waitTime.Text)

			Dim msgBuilder As New StringBuilder
			For i  = 0 To bytesPerMsg - 1
				msgBuilder.Append("A")
			Next i

			LoadTestComet(maxUsers, repsPerSet, msgBuilder.ToString, waitTime)
		End Sub

		Public Sub LoadTestComet(ByVal maxNumberOfUsers As Integer, ByVal repsPerSet As Integer, ByVal message As String, ByVal waitTime As Integer)
			If maxNumberOfUsers <= 0 OrElse repsPerSet <= 0 Then
				Exit Sub
			End If

			Dim doneEvents(maxNumberOfUsers - 1) As ManualResetEvent
			Dim cometTester(maxNumberOfUsers - 1) As CometTester

			For numOfUsers  = 0 To maxNumberOfUsers - 1
				doneEvents(numOfUsers) = New ManualResetEvent(False)
				Dim t As New CometTester(numOfUsers, doneEvents(numOfUsers), message, repsPerSet, waitTime)
				cometTester(numOfUsers) = t
				ThreadPool.QueueUserWorkItem(AddressOf t.EngageCometBroadcast, numOfUsers)
			Next numOfUsers

            'WaitHandle.WaitAll(doneEvents)
			For Each resetEvent As ManualResetEvent In doneEvents
				resetEvent.WaitOne()
			Next resetEvent

			Dim totalTime As Double = 0
			Dim numberOfFailures As Integer = 0
			Dim numberOfWcfFailures As Integer = 0

			For i  = 0 To maxNumberOfUsers - 1
				Dim t As CometTester = cometTester(i)
				totalTime += t.TestResults.TimeTakenInMilliseconds
				numberOfFailures += t.TestResults.RepsFailed
				numberOfWcfFailures += t.TestResults.WcfFailures
			Next i

			Dim averageResponseTime As Double = totalTime / maxNumberOfUsers

			MessageBox.Show("The average Comet response time was " & averageResponseTime & " ms.  Number of failures: " & numberOfFailures & ".  Number of WCF failures: " & numberOfWcfFailures)
		End Sub

		Private Sub _pingLoadTestButton_Click(ByVal sender As Object, ByVal e As EventArgs) Handles _pingLoadTestButton.Click
			Dim maxUsers As Integer = Convert.ToInt32(_maxUserLoad.Text)
			Dim repsPerSet As Integer = Convert.ToInt32(_repsPerLoadSet.Text)
            'Dim bytesPerMsg As Integer = Convert.ToInt32(_bytesPerMessage.Text)
            Dim waitTime As Integer = Convert.ToInt32(_waitTime.Text)

			PingComet(maxUsers, repsPerSet, "PING", waitTime)
		End Sub

		Public Sub PingComet(ByVal maxNumberOfUsers As Integer, ByVal repsPerSet As Integer, ByVal message As String, ByVal waitTime As Integer)
			If maxNumberOfUsers <= 0 OrElse repsPerSet <= 0 Then
				Exit Sub
			End If

			Dim doneEvents(maxNumberOfUsers - 1) As ManualResetEvent
			Dim cometTester(maxNumberOfUsers - 1) As CometTester

			For numOfUsers  = 0 To maxNumberOfUsers - 1
				doneEvents(numOfUsers) = New ManualResetEvent(False)
				Dim t As New CometTester(numOfUsers, doneEvents(numOfUsers), message, repsPerSet, waitTime)
				cometTester(numOfUsers) = t
				ThreadPool.QueueUserWorkItem(AddressOf t.PingComet, numOfUsers)
			Next numOfUsers

            'WaitHandle.WaitAll(doneEvents)
			For Each resetEvent As ManualResetEvent In doneEvents
				resetEvent.WaitOne()
			Next resetEvent

			Dim totalTime As Double = 0
			Dim numberOfFailures As Integer = 0
			Dim numberOfWcfFailures As Integer = 0

			For i  = 0 To maxNumberOfUsers - 1
				Dim t As CometTester = cometTester(i)
				totalTime += t.TestResults.TimeTakenInMilliseconds
				numberOfFailures += t.TestResults.RepsFailed
				numberOfWcfFailures += t.TestResults.WcfFailures
			Next i

			Dim averageResponseTime As Double = totalTime / maxNumberOfUsers

			MessageBox.Show("The average Comet response time was " & averageResponseTime & " ms.  Number of failures: " & numberOfFailures & ".  Number of WCF failures: " & numberOfWcfFailures)
		End Sub

		Private Sub _broadcastTestButton_Click(ByVal sender As Object, ByVal e As EventArgs) Handles _broadcastTestButton.Click
			Dim maxUsers As Integer = Convert.ToInt32(_maxUserLoad.Text)
			_totalUsers = maxUsers
			Dim repsPerSet As Integer = Convert.ToInt32(_repsPerLoadSet.Text)
			Dim bytesPerMsg As Integer = Convert.ToInt32(_bytesPerMessage.Text)
			Dim waitTime As Integer = Convert.ToInt32(_waitTime.Text)

			Dim messageBuilder As New StringBuilder

			For i  = 0 To bytesPerMsg - 1
				messageBuilder.Append("B")
			Next i

			_message = messageBuilder.ToString
			broadcastTestComet(maxUsers, repsPerSet, messageBuilder.ToString, waitTime)
		End Sub

		Public Sub broadcastTestComet(ByVal maxNumberOfUsers As Integer, ByVal repsPerSet As Integer, ByVal message As String, ByVal waitTime As Integer)
			If maxNumberOfUsers <= 0 OrElse repsPerSet <= 0 Then
				Exit Sub
			End If

			Dim doneEvents(maxNumberOfUsers - 1) As ManualResetEvent
			Dim cometTester(maxNumberOfUsers - 1) As CometTester

			For numOfUsers  = 0 To maxNumberOfUsers - 1
				doneEvents(numOfUsers) = New ManualResetEvent(False)
				Dim t As New CometTester(numOfUsers, doneEvents(numOfUsers), message, repsPerSet, waitTime)
				AddHandler t.CometConnectionEvent, AddressOf connectedToComet
				cometTester(numOfUsers) = t
				ThreadPool.QueueUserWorkItem(AddressOf t.AsyncTest, numOfUsers)
			Next numOfUsers

            'Dim sendMessageResetEvent As New ManualResetEvent(False)
            'Dim t1 As New CometTester(1, sendMessageResetEvent, message, repsPerSet, waitTime)
            'ThreadPool.QueueUserWorkItem(t1.DumpMessage, maxNumberOfUsers)

			'Dump message to message queue:

            'WaitHandle.WaitAll(doneEvents)
			For Each resetEvent As ManualResetEvent In doneEvents
				resetEvent.WaitOne()
			Next resetEvent

            'sendMessageResetEvent.WaitOne

			Dim totalTime As Double = 0
			Dim numberOfFailures As Integer = 0
			Dim numberOfWcfFailures As Integer = 0

			For i  = 0 To maxNumberOfUsers - 1
				Dim t As CometTester = cometTester(i)
				totalTime += t.TestResults.TimeTakenInMilliseconds
				numberOfFailures += t.TestResults.RepsFailed
				numberOfWcfFailures += t.TestResults.WcfFailures
			Next i

			Dim averageResponseTime As Double = totalTime / (maxNumberOfUsers * repsPerSet - numberOfFailures)

			MessageBox.Show("The average Comet response time was " & averageResponseTime & " ms.  Number of failures: " & numberOfFailures & ".  Number of WCF failures: " & numberOfWcfFailures)
		End Sub

		Private Sub connectedToComet(ByVal sender As Object, ByVal args As CometConnectionEventArgs)
			_numOfConnections += 1
			If _numOfConnections > _totalUsers Then
				sendMessage()
			End If
		End Sub

		Private Sub sendMessage()
			'Dump message to message queue:
			Try
                '_cometServer.BroadcastMessage(_sendCometMsgTextBox.Text)
				Dim msgQueueName As String = ConfigurationManager.AppSettings("CometQueue")
				Dim testMessageQueue As New MessageQueue(msgQueueName)
				Dim msg As New Messaging.Message(_message, New XmlMessageFormatter(New Type() { GetType(String) }))
				msg.Body = _message
				testMessageQueue.Send(msg)
			Catch ex As Exception
                Dim err As String = ex.ToString
                'Response.Write(error)
				Throw ex
			End Try
		End Sub
	End Class
End Namespace

